@@ -36,6 +36,7 @@ from .model_version import ModelVersion
 from .modification_lock_info import ModificationLockInfo
 from .nested_column import NestedColumn
 from .organization_group import OrganizationGroup
+from .point_in_time_feature import PointInTimeFeature
 from .prediction_dataset import PredictionDataset
 from .prediction_feature_group import PredictionFeatureGroup
 from .prediction_input import PredictionInput
@@ -55,6 +56,12 @@ from .use_case_requirements import UseCaseRequirements
 from .user import User
 
 
+class ClientOptions:
+    def __init__(self, exception_on_404=True, server='https://abacus.ai'):
+        self.exception_on_404 = exception_on_404
+        self.server = server
+
+
 class ApiException(Exception):
     def __init__(self, message, http_status, exception=None):
         self.message = message
@@ -66,12 +73,13 @@ class ApiException(Exception):
 
 
 class ApiClient():
-    client_version = '0.30.2'
+    client_version = '0.30.8'
 
-    def __init__(self, api_key=None, server='https://abacus.ai'):
+    def __init__(self, api_key: str = None, server: str = None, client_options: ClientOptions = None):
         self.api_key = api_key
         self.web_version = None
-        self.server = server
+        self.client_options = client_options or ClientOptions()
+        self.server = server or self.client_options.server
         self.user = None
         # Connection and version check
         try:
@@ -119,6 +127,8 @@ class ApiClient():
         if not success:
             if response.status_code > 502 and response.status_code not in (501, 503):
                 error_message = 'Internal Server Error, please contact dev@abacus.ai for support'
+            if response.status_code == 404 and not self.client_options.exception_on_404:
+                return None
             raise ApiException(error_message, response.status_code, error_type)
         return result
 
@@ -376,7 +386,7 @@ class ApiClient():
         '''Delete a nested feature.'''
         return self._call_api('deleteNestedFeature', 'DELETE', query_params={'featureGroupId': feature_group_id, 'nestedFeatureName': nested_feature_name}, parse_type=FeatureGroup)
 
-    def create_point_in_time_feature(self, feature_group_id: str, feature_name: str, history_table_name: str = None, aggregation_key_features: list = None, time_feature: str = None, historical_time_feature: str = None, lookback_window_seconds: float = None, lookback_window_lag_seconds: float = 0, lookback_count: int = None, lookback_until_position: int = 0, expression: str = None) -> FeatureGroup:
+    def create_point_in_time_feature(self, feature_group_id: str, feature_name: str, history_table_name: str = None, aggregation_keys: list = None, timestamp_key: str = None, historical_timestamp_key: str = None, lookback_window_seconds: float = None, lookback_window_lag_seconds: float = 0, lookback_count: int = None, lookback_until_position: int = 0, expression: str = None) -> FeatureGroup:
         '''Creates a new point in time feature in a feature group using another historical feature group, window spec and aggregate expression.
 
         We use the aggregation keys, and either the lookbackWindowSeconds or the lookbackCount values to perform the window aggregation for every row in the current feature group.
@@ -386,11 +396,11 @@ class ApiClient():
         a lookup on this feature group. If window is specified in counts, then we order the historical table rows aligning by time and consider rows from the window where
         the rank order is >= lookbackCount and includes the row just prior to the current one. The lag is specified in term of positions using lookbackUntilPosition.
         '''
-        return self._call_api('createPointInTimeFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'featureName': feature_name, 'historyTableName': history_table_name, 'aggregationKeyFeatures': aggregation_key_features, 'timeFeature': time_feature, 'historicalTimeFeature': historical_time_feature, 'lookbackWindowSeconds': lookback_window_seconds, 'lookbackWindowLagSeconds': lookback_window_lag_seconds, 'lookbackCount': lookback_count, 'lookbackUntilPosition': lookback_until_position, 'expression': expression}, parse_type=FeatureGroup)
+        return self._call_api('createPointInTimeFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'featureName': feature_name, 'historyTableName': history_table_name, 'aggregationKeys': aggregation_keys, 'timestampKey': timestamp_key, 'historicalTimestampKey': historical_timestamp_key, 'lookbackWindowSeconds': lookback_window_seconds, 'lookbackWindowLagSeconds': lookback_window_lag_seconds, 'lookbackCount': lookback_count, 'lookbackUntilPosition': lookback_until_position, 'expression': expression}, parse_type=FeatureGroup)
 
-    def update_point_in_time_feature(self, feature_group_id: str, feature_name: str, history_table_name: str = None, aggregation_key_features: list = None, time_feature: str = None, historical_time_feature: str = None, lookback_window_seconds: float = None, lookback_window_lag_seconds: float = 0, lookback_count: int = None, lookback_until_position: int = 0, expression: str = None, new_feature_name: str = None) -> FeatureGroup:
+    def update_point_in_time_feature(self, feature_group_id: str, feature_name: str, history_table_name: str = None, aggregation_keys: list = None, timestamp_key: str = None, historical_timestamp_key: str = None, lookback_window_seconds: float = None, lookback_window_lag_seconds: float = 0, lookback_count: int = None, lookback_until_position: int = 0, expression: str = None, new_feature_name: str = None) -> FeatureGroup:
         '''Updates an existing point in time feature in a feature group. See createPointInTimeFeature for detailed semantics.'''
-        return self._call_api('updatePointInTimeFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'featureName': feature_name, 'historyTableName': history_table_name, 'aggregationKeyFeatures': aggregation_key_features, 'timeFeature': time_feature, 'historicalTimeFeature': historical_time_feature, 'lookbackWindowSeconds': lookback_window_seconds, 'lookbackWindowLagSeconds': lookback_window_lag_seconds, 'lookbackCount': lookback_count, 'lookbackUntilPosition': lookback_until_position, 'expression': expression, 'newFeatureName': new_feature_name}, parse_type=FeatureGroup)
+        return self._call_api('updatePointInTimeFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'featureName': feature_name, 'historyTableName': history_table_name, 'aggregationKeys': aggregation_keys, 'timestampKey': timestamp_key, 'historicalTimestampKey': historical_timestamp_key, 'lookbackWindowSeconds': lookback_window_seconds, 'lookbackWindowLagSeconds': lookback_window_lag_seconds, 'lookbackCount': lookback_count, 'lookbackUntilPosition': lookback_until_position, 'expression': expression, 'newFeatureName': new_feature_name}, parse_type=FeatureGroup)
 
     def attach_feature_group_to_project(self, feature_group_id: str, project_id: str, feature_group_type: str = 'CUSTOM_TABLE'):
         '''[DEPRECATED] Adds a feature group to a project,'''
@@ -422,11 +432,11 @@ class ApiClient():
 
     def invalidate_streaming_feature_group_data(self, feature_group_id: str, invalid_before_timestamp: int):
         '''Invalidates all streaming data with timestamp before invalidBeforeTimestamp'''
-        return self._call_api('invalidateStreamingFeatureGroupData', 'GET', query_params={'featureGroupId': feature_group_id, 'invalidBeforeTimestamp': invalid_before_timestamp})
+        return self._call_api('invalidateStreamingFeatureGroupData', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'invalidBeforeTimestamp': invalid_before_timestamp})
 
-    def concatenate_feature_group_data(self, feature_group_id: str, source_feature_group_id: str, merge_type: str = 'UNION', after_timestamp: int = None):
-        '''Concatenating streaming feature group with offline data Streaming feature groups can be merged with a regular feature group using a concatenate operation. Feature groups can be merged if their schema's are compatible and they have the special recordTimestamp column and if set, the recordId column. The second operand in the concatenate operation will be appended to the first operand (merge target).'''
-        return self._call_api('concatenateFeatureGroupData', 'GET', query_params={'featureGroupId': feature_group_id, 'sourceFeatureGroupId': source_feature_group_id, 'mergeType': merge_type, 'afterTimestamp': after_timestamp})
+    def concatenate_feature_group_data(self, feature_group_id: str, source_feature_group_id: str, merge_type: str = 'UNION', replace_until_timestamp: int = None):
+        '''Concatenates data from one feature group to another. Feature groups can be merged if their schema's are compatible and they have the special updateTimestampKey column and if set, the primaryKey column. The second operand in the concatenate operation will be appended to the first operand (merge target).'''
+        return self._call_api('concatenateFeatureGroupData', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'sourceFeatureGroupId': source_feature_group_id, 'mergeType': merge_type, 'replaceUntilTimestamp': replace_until_timestamp})
 
     def describe_feature_group(self, feature_group_id: str) -> FeatureGroup:
         '''Describe a Feature Group.'''
@@ -436,11 +446,11 @@ class ApiClient():
         '''Describe a Feature Group by the feature group's table name'''
         return self._call_api('describeFeatureGroupByTableName', 'GET', query_params={'tableName': table_name}, parse_type=FeatureGroup)
 
-    def set_feature_group_record_attributes(self, feature_group_id: str, record_id_feature: str = None, record_timestamp_feature: str = None, lookup_keys: list = None):
+    def set_feature_group_indexing_config(self, feature_group_id: str, primary_key: str = None, update_timestamp_key: str = None, lookup_keys: list = None):
         '''Sets various attributes of the feature group used for deployment lookups and streaming updates.'''
-        return self._call_api('setFeatureGroupRecordAttributes', 'GET', query_params={'featureGroupId': feature_group_id, 'recordIdFeature': record_id_feature, 'recordTimestampFeature': record_timestamp_feature, 'lookupKeys': lookup_keys})
+        return self._call_api('setFeatureGroupIndexingConfig', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'primaryKey': primary_key, 'updateTimestampKey': update_timestamp_key, 'lookupKeys': lookup_keys})
 
-    def list_feature_groups(self, limit: int = 100, start_after_id: str = None) -> FeatureGroup:
+    def list_feature_groups(self, limit: int = 100, start_after_id: str = None) -> List[FeatureGroup]:
         '''Enlist all the feature groups associated with a project. A user needs to specify the unique project ID to fetch all attached feature groups.'''
         return self._call_api('listFeatureGroups', 'GET', query_params={'limit': limit, 'startAfterId': start_after_id}, parse_type=FeatureGroup)
 
@@ -465,9 +475,21 @@ class ApiClient():
         return self._call_api('updateFeature', 'PATCH', query_params={}, body={'featureGroupId': feature_group_id, 'name': name, 'selectExpression': select_expression, 'newName': new_name}, parse_type=FeatureGroup)
 
     def export_feature_group_version_to_file_connector(self, feature_group_version: str, location: str, export_file_format: str) -> FeatureGroupExport:
-        ''''''
+        '''Export Feature group to File Connector.'''
         return self._call_api('exportFeatureGroupVersionToFileConnector', 'POST', query_params={}, body={'featureGroupVersion': feature_group_version, 'location': location, 'exportFileFormat': export_file_format}, parse_type=FeatureGroupExport)
 
+    def export_feature_group_version_to_database_connector(self, feature_group_version: str, database_connector_id: str, object_name: str, write_mode: str, database_feature_mapping: dict, id_column: str) -> FeatureGroupExport:
+        '''Export Feature group to Database Connector.'''
+        return self._call_api('exportFeatureGroupVersionToDatabaseConnector', 'POST', query_params={}, body={'featureGroupVersion': feature_group_version, 'databaseConnectorId': database_connector_id, 'objectName': object_name, 'writeMode': write_mode, 'databaseFeatureMapping': database_feature_mapping, 'idColumn': id_column}, parse_type=FeatureGroupExport)
+
+    def describe_feature_group_export(self, feature_group_export_id: str) -> FeatureGroupExport:
+        '''A feature group export'''
+        return self._call_api('describeFeatureGroupExport', 'GET', query_params={'featureGroupExportId': feature_group_export_id}, parse_type=FeatureGroupExport)
+
+    def list_feature_group_exports(self, feature_group_id: str) -> List[FeatureGroupExport]:
+        '''Lists all of the feature group exports for a given feature group'''
+        return self._call_api('listFeatureGroupExports', 'GET', query_params={'featureGroupId': feature_group_id}, parse_type=FeatureGroupExport)
+
     def set_feature_group_modifier_lock(self, feature_group_id: str, locked: bool = True):
         ''''''
         return self._call_api('setFeatureGroupModifierLock', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'locked': locked})
@@ -913,9 +935,9 @@ class ApiClient():
         ''''''
         return self._call_api('getFeatureGroupRows', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data})
 
-    def create_batch_prediction(self, deployment_id: str, name: str = None, global_prediction_args: dict = None, explanations: bool = False, output_format: str = None, output_location: str = None, database_connector_id: str = None, database_output_config: dict = None, refresh_schedule: str = None) -> BatchPrediction:
+    def create_batch_prediction(self, deployment_id: str, name: str = None, global_prediction_args: dict = None, explanations: bool = False, output_format: str = None, output_location: str = None, database_connector_id: str = None, database_output_config: dict = None, refresh_schedule: str = None, csv_input_prefix: str = None, csv_prediction_prefix: str = None, csv_explanations_prefix: str = None) -> BatchPrediction:
         '''Creates a batch prediction job description for the given deployment.'''
-        return self._call_api('createBatchPrediction', 'POST', query_params={'deploymentId': deployment_id}, body={'name': name, 'globalPredictionArgs': global_prediction_args, 'explanations': explanations, 'outputFormat': output_format, 'outputLocation': output_location, 'databaseConnectorId': database_connector_id, 'databaseOutputConfig': database_output_config, 'refreshSchedule': refresh_schedule}, parse_type=BatchPrediction)
+        return self._call_api('createBatchPrediction', 'POST', query_params={'deploymentId': deployment_id}, body={'name': name, 'globalPredictionArgs': global_prediction_args, 'explanations': explanations, 'outputFormat': output_format, 'outputLocation': output_location, 'databaseConnectorId': database_connector_id, 'databaseOutputConfig': database_output_config, 'refreshSchedule': refresh_schedule, 'csvInputPrefix': csv_input_prefix, 'csvPredictionPrefix': csv_prediction_prefix, 'csvExplanationsPrefix': csv_explanations_prefix}, parse_type=BatchPrediction)
 
     def start_batch_prediction(self, batch_prediction_id: str) -> BatchPredictionVersion:
         '''Creates a new batch prediction version job for a given batch prediction job description'''
@@ -951,9 +973,9 @@ class ApiClient():
         '''Describes a batch prediction version'''
         return self._call_api('describeBatchPredictionVersion', 'GET', query_params={'batchPredictionVersion': batch_prediction_version}, parse_type=BatchPredictionVersion)
 
-    def update_batch_prediction(self, batch_prediction_id: str, deployment_id: str = None, global_prediction_args: dict = None, explanations: bool = None, output_format: str = None) -> BatchPrediction:
+    def update_batch_prediction(self, batch_prediction_id: str, deployment_id: str = None, global_prediction_args: dict = None, explanations: bool = None, output_format: str = None, csv_input_prefix: str = None, csv_prediction_prefix: str = None, csv_explanations_prefix: str = None) -> BatchPrediction:
         '''Updates a batch prediction job description'''
-        return self._call_api('updateBatchPrediction', 'POST', query_params={'deploymentId': deployment_id}, body={'batchPredictionId': batch_prediction_id, 'globalPredictionArgs': global_prediction_args, 'explanations': explanations, 'outputFormat': output_format}, parse_type=BatchPrediction)
+        return self._call_api('updateBatchPrediction', 'POST', query_params={'deploymentId': deployment_id}, body={'batchPredictionId': batch_prediction_id, 'globalPredictionArgs': global_prediction_args, 'explanations': explanations, 'outputFormat': output_format, 'csvInputPrefix': csv_input_prefix, 'csvPredictionPrefix': csv_prediction_prefix, 'csvExplanationsPrefix': csv_explanations_prefix}, parse_type=BatchPrediction)
 
     def set_batch_prediction_file_connector_output(self, batch_prediction_id: str, output_format: str = None, output_location: str = None) -> BatchPrediction:
         '''Updates the file connector output configuration of the batch prediction'''
@@ -963,6 +985,10 @@ class ApiClient():
         '''Updates the database connector output configuration of the batch prediction'''
         return self._call_api('setBatchPredictionDatabaseConnectorOutput', 'POST', query_params={}, body={'batchPredictionId': batch_prediction_id, 'databaseConnectorId': database_connector_id, 'databaseOutputConfig': database_output_config}, parse_type=BatchPrediction)
 
+    def set_batch_prediction_feature_group_output(self, batch_prediction_id: str, table_name: str) -> BatchPrediction:
+        '''Creates a feature group and sets it to be the batch prediction output'''
+        return self._call_api('setBatchPredictionFeatureGroupOutput', 'POST', query_params={}, body={'batchPredictionId': batch_prediction_id, 'tableName': table_name}, parse_type=BatchPrediction)
+
     def set_batch_prediction_output_to_console(self, batch_prediction_id: str) -> BatchPrediction:
         '''Sets the batch prediction output to the console, clearing both the file connector and database connector config'''
         return self._call_api('setBatchPredictionOutputToConsole', 'POST', query_params={}, body={'batchPredictionId': batch_prediction_id}, parse_type=BatchPrediction)
